@@ -12,8 +12,8 @@
 #' @keywords internal
 #' @family hidden helper functions for \code{\link{htmlTable}}
 prTblNo <- function (caption) {
-  tc <- getOption("table_counter")
-  if (is.null(tc)){
+  tc <- getOption("table_counter", FALSE)
+  if (tc == FALSE){
     if (missing(caption))
       return("")
     else
@@ -92,7 +92,7 @@ prGetStyle <- function(...){
                    names(dots)[i] != ""){
           element <-
             paste0(names(dots)[i], ": ", element)
-        }else{
+        }else if(element != "none") {
           stop("The style should be formatted according to 'style_name: value'",
                " you have provided style '", element,"'")
         }
@@ -167,8 +167,12 @@ prAddSemicolon2StrEnd <- function(my_str){
   if(length(my_str) == 0)
     return("")
 
-  if (tail(strsplit(my_str, "")[[1]], 1) != ";")
+  if (tail(strsplit(my_str, "")[[1]], 1) != ";"){
+    n <- names(my_str)
     my_str <- sprintf("%s;", my_str)
+    if (!is.null(n))
+      names(my_str) <- n
+  }
 
   # Remove duplicated ;
   my_str <- gsub(";;+", ";", my_str)
@@ -656,11 +660,12 @@ prSkipRownames <- function(rnames){
 #'
 #' @param clr The colors
 #' @param n The number of rows/columns applicable to the color
-#' @param ng The n.rgroup argument if applicable
+#' @param ng The n.rgroup/n.cgroup argument if applicable
+#' @param gtxt The rgroup/cgroup texts
 #' @return \code{character} A vector containing hexadecimal colors
 #' @import magrittr
 #' @keywords internal
-prPrepareColors <- function(clr, n, ng){
+prPrepareColors <- function(clr, n, ng, gtxt){
   clr <- sapply(clr, function(a_clr){
     if(a_clr == "none")
       return(a_clr)
@@ -684,8 +689,23 @@ prPrepareColors <- function(clr, n, ng){
   }, USE.NAMES=FALSE)
 
   if(!missing(ng)){
-    clr <- rep(clr, length.out = length(ng))
+    # Split groups into separate if the gtxt is ""
+    if (any(gtxt == "")){
+      tmp <- c()
+      for (i in 1:length(ng)){
+        if (gtxt[i] != "" &&
+              !is.na(gtxt[i])){
+          tmp <- c(tmp,
+                   ng[i])
+        }else{
+          tmp <- c(tmp,
+                   rep(1, ng[i]))
+        }
+      }
+      ng <- tmp
+    }
 
+    clr <- rep(clr, length.out = length(ng))
     attr(clr, "groups") <-
       Map(rep, clr, length.out = ng)
   }else if(!missing(n)){
@@ -812,3 +832,51 @@ prPrepareCss <- function(x, css, rnames, header, name = deparse(substitute(css))
                    header = css.header,
                    class=class(css)))
 }
+
+
+#' Get the add attribute element
+#'
+#' Gets the add element attribute if it exists. If non-existant it will
+#' return NULL.
+#'
+#' @param rgroup_iterator The rgroup number of interest
+#' @inheritParams htmlTable
+#' @keywords internal
+prAttr4RgroupAdd <- function (rgroup, rgroup_iterator) {
+  if (is.null(attr(rgroup, "add")))
+    return(NULL)
+
+  add_elmnt <- attr(rgroup, "add")
+  if (is.null(names(add_elmnt))){
+    if (length(add_elmnt) != sum(rgroup !=  ""))
+      stop("The length of the rgroup 'add' attribute must either match",
+           " (1) the length of the rgroup",
+           " (2) or have names corresponding to the mapping integers")
+
+    names(add_elmnt) <- (1:length(rgroup))[rgroup !=  ""]
+  }
+
+  if (!is.list(add_elmnt) &&
+        !is.vector(add_elmnt))
+    stop("The rgroup mus either be a list or a vector")
+
+  add_pos <- as.integer(names(add_elmnt))
+  if (any(is.na(add_pos)))
+    stop("The rgroup 'add' element contains invalid names: ",
+         "'", paste(names(add_elmnt)[is.na(add_pos)], collapse="', '"), "'")
+
+  if (any(add_pos < 1))
+    stop("The rgroup 'add' attribute cannot have integer names below 1")
+
+  if (any(add_pos > length(rgroup)))
+    stop("The rgroup 'add' attribute cannot have integer names indicating",
+         " positions larger than the length of the rgroup ('", length(rgroup), "').",
+         " The problematic position(s):",
+         " '", paste(add_pos[add_pos > length(rgroup)], collapse="', '") ,"'")
+
+  if (rgroup_iterator %in% names(add_elmnt)){
+    return(add_elmnt[[as.character(rgroup_iterator)]])
+  }
+
+  return(NULL)
+}
